@@ -116,6 +116,8 @@ void cleanup_context(isel_context* ctx);
 
 isel_context setup_isel_context(Program* program, unsigned shader_count,
                                 struct nir_shader* const* shaders, ac_shader_config* config,
+                                const struct radv_nir_compiler_options* options,
+                                const struct radv_shader_info* info,
                                 const struct radv_shader_args* args, bool is_gs_copy_shader);
 
 } // namespace aco
