@@ -8,10 +8,6 @@
 #include "frontend/sw_winsys.h"
 #include "target-helpers/inline_debug_helper.h"
 
-#ifdef GALLIUM_SWR
-#include "swr/swr_public.h"
-#endif
-
 /* Helper function to choose and instantiate one of the software rasterizers:
  * llvmpipe, softpipe.
  */
@@ -33,6 +29,10 @@
 #include "d3d12/d3d12_public.h"
 #endif
 
+#ifdef GALLIUM_ASAHI
+#include "asahi/agx_public.h"
+#endif
+
 static inline struct pipe_screen *
 sw_screen_create_named(struct sw_winsys *winsys, const char *driver)
 {
@@ -56,11 +56,6 @@ sw_screen_create_named(struct sw_winsys *winsys, const char *driver)
       screen = softpipe_create_screen(winsys);
 #endif
 
-#if defined(GALLIUM_SWR)
-   if (screen == NULL && strcmp(driver, "swr") == 0)
-      screen = swr_create_screen(winsys);
-#endif
-
 #if defined(GALLIUM_ZINK)
    if (screen == NULL && strcmp(driver, "zink") == 0)
       screen = zink_create_screen(winsys);
@@ -71,30 +66,35 @@ sw_screen_create_named(struct sw_winsys *winsys, const char *driver)
       screen = d3d12_create_dxcore_screen(winsys, NULL);
 #endif
 
+#if defined(GALLIUM_ASAHI)
+   if (screen == NULL && strcmp(driver, "asahi") == 0)
+      screen = agx_screen_create(winsys);
+#endif
+
    return screen ? debug_screen_wrap(screen) : NULL;
 }
 
 
 static inline struct pipe_screen *
-sw_screen_create(struct sw_winsys *winsys)
+sw_screen_create_vk(struct sw_winsys *winsys, bool sw_vk)
 {
    UNUSED bool only_sw = env_var_as_boolean("LIBGL_ALWAYS_SOFTWARE", false);
    const char *drivers[] = {
-      debug_get_option("GALLIUM_DRIVER", ""),
+      (sw_vk ? "" : debug_get_option("GALLIUM_DRIVER", "")),
 #if defined(GALLIUM_D3D12)
-      only_sw ? "" : "d3d12",
+      (sw_vk || only_sw) ? "" : "d3d12",
+#endif
+#if defined(GALLIUM_ASAHI)
+      (sw_vk || only_sw) ? "" : "asahi",
 #endif
 #if defined(GALLIUM_LLVMPIPE)
       "llvmpipe",
 #endif
 #if defined(GALLIUM_SOFTPIPE)
-      "softpipe",
-#endif
-#if defined(GALLIUM_SWR)
-      "swr",
+      (sw_vk ? "" : "softpipe"),
 #endif
 #if defined(GALLIUM_ZINK)
-      only_sw ? "" : "zink",
+      (sw_vk || only_sw) ? "" : "zink",
 #endif
    };
 
@@ -109,4 +109,9 @@ sw_screen_create(struct sw_winsys *winsys)
    return NULL;
 }
 
+static inline struct pipe_screen *
+sw_screen_create(struct sw_winsys *winsys)
+{
+   return sw_screen_create_vk(winsys, false);
+}
 #endif
